
package shai.vmutils;

import java.io.*;
import java.util.Vector;
import shai.vmutils.AttributeNode;

/**
 * This class represents the attributes of fields methods and the class itself.
**/
public class AttributeEntry 
{
	// this list contains all of the attributes.
	Vector attributeList;

/**
 * This method was created by a SmartGuide.
 * @param dataStream java.io.DataInputStream
 * @exception java.io.IOException The exception description.
 * @exception java.lang.ClassFormatError The exception description.
 */
public AttributeEntry (DataInputStream dataStream) 
		throws IOException, ClassFormatError 
{
	attributeList = new Vector();
	addEntry(dataStream);
}
/**
 * This method fills the attributeList with the AttributeNode instances from the dataStream,
 * @param dataStream DataInputStream
 * @param ConstantPoolEntry constantPool[]
 * @exception java.io.IOException The exception description.
 */
public AttributeEntry(DataInputStream dataStream, ConstantPoolEntry constantPool[]) throws java.io.IOException, ClassFormatError
{ 
	int attributeNumber = dataStream.readShort();
	if(attributeNumber > 0)
		attributeList = new Vector();
	else
		attributeList = null;

	// iterate through the file and load all the attributes to the Vector.	
	for(int iter = 0 ; iter < attributeNumber ; iter++)
	{
		addEntry(dataStream);
	}	
}
/**
 * This method was created by a SmartGuide.
 * @param dataStream java.io.DataInputStream
 * @exception java.io.IOException The exception description.
 * @exception java.lang.ClassFormatError The exception description.
 */
public void addEntry (DataInputStream dataStream) 
		throws IOException, ClassFormatError 
{
	// the data related to the attribte.
	byte data[];

	// the name of the attribute and its length.
	int name = dataStream.readShort();
	int length = dataStream.readInt();

	if(length != 0)
	{
		data = new byte[length];
		dataStream.readFully(data);

		attributeList.addElement(new AttributeNode(name, data));
	}	
}
/**
 * This method returns the attribute at offset.
 * @return shai.vmutils.AttributeNode
 * @param offset int
 */
public AttributeNode getAttribute(int offset) 
{
	return ((AttributeNode)attributeList.elementAt(offset));

}
/**
 * This method returns the number of attributes that are related to this instance.
 * @return int
 */
public int getAttributeCount() 
{
	if(attributeList == null) 
		return(0);
	return (attributeList.size());
}
/**
 * This method was created by a SmartGuide.
 * @param outputStream java.io.DataOutputStream
 * @exception java.io.IOException The exception description.
 */
public void save(DataOutputStream outputStream) 
		throws IOException 
{
	if(attributeList == null)
	{
		outputStream.writeShort(0);
		return;
	}	
		
	outputStream.writeShort(attributeList.size());
	
	AttributeNode currentAttribute;
		
	// iterate through the file and load all the attributes to the Vector.	
	for(int iter = 0 ; iter < attributeList.size() ; iter++)
	{
		currentAttribute = (AttributeNode)attributeList.elementAt(iter);

		// write the name of the attribute and its length.
		outputStream.writeShort(currentAttribute.getName());
		outputStream.writeInt(currentAttribute.getData().length);

		if(currentAttribute.getData().length != 0)
		{
			outputStream.write(currentAttribute.getData(), 0, currentAttribute.getData().length);
		}	
	}	
}
}