
package shai.vmutils;

import java.io.*;
import shai.vmutils.AttributeEntry;

/**
 * This class contains the representation of a field in the class.
 */
public class FieldEntry 
{
	// the offset in the constant pool containing the field name.
	int fieldName;
	
	// the offset in the constant pool containing the field type.
	int fieldType;

	// the attributes of the field.
	AttributeEntry fieldAttributes;

	int fieldMode;

/**
 * This method loads the field into memory from the class file.
 * @param dataStream java.io.DataInputStream
 * @param ConstantPoolEntry constantPool[]
 * @exception java.io.IOException The exception description.
 */
public FieldEntry (DataInputStream dataStream, ConstantPoolEntry constantPool[]) throws IOException, ClassFormatError
{
	fieldMode = dataStream.readShort();
	
	// the offset in the constant pool containing the field name.
	fieldName = dataStream.readShort();

	// the offset in the constant pool containing the field type.
	fieldType = dataStream.readShort();
	
	// the attributes of the field.
	fieldAttributes = new AttributeEntry(dataStream, constantPool);
}
/**
 * This method returns the fields attributes.
 * @return shai.vmutils.AttributeEntry
 */
public AttributeEntry getFieldAttributes() 
{
	return (fieldAttributes);
}
/**
 * This method returns the offset of the field name in the constant pool.
 * @return java.lang.String
 */
public int getFieldName() 
{
	return (fieldName);
}
/**
 * This method returns the offset of the field type in the constant pool.
 * @return java.lang.String
 */
public int getFieldType() 
{
	return (fieldType);
}
/**
 * This method was created by a SmartGuide.
 * @param outputStream java.io.DataOutputStream
 * @exception java.io.IOException The exception description.
 */
public void save(DataOutputStream outputStream) 
		throws IOException 
{
	outputStream.writeShort(fieldMode);
	
	// the offset in the constant pool containing the field name.
	outputStream.writeShort(fieldName);

	// the offset in the constant pool containing the field type.
	outputStream.writeShort(fieldType);
	
	// the attributes of the field.
	fieldAttributes.save(outputStream);

}
}