
package shai.vmutils;

import java.io.*;
import shai.vmutils.AttributeEntry;

/**
 * This class contains the representation of a method in the class file.
**/
public class MethodEntry 
{
	// the offset in the constant pool containing the method name.
	int methodName;
	
	// the offset in the constant pool containing the method type.
	int methodType;

	// the attributes of the method.
	AttributeEntry methodAttributes;

	int methodMode;

/**
 * This method reads the data regarding the method into memory.
 * @param dataStream java.io.DataInputStream
 * @param ConstantPoolEntry constantPool[]
 * @exception java.io.IOException The exception description.
 */
public MethodEntry (DataInputStream dataStream, ConstantPoolEntry constantPool[]) throws IOException, ClassFormatError
{
	methodMode = dataStream.readShort();
	
	// the offset in the constant pool containing the method name.
	methodName = dataStream.readShort();

	// the offset in the constant pool containing the method type.
	methodType = dataStream.readShort();

	// the attributes of the method.
	methodAttributes = new AttributeEntry(dataStream, constantPool);
}
/**
 * This method returns the methods attributes.
 * @return shai.vmutils.AttributeEntry
 */
public AttributeEntry getMethodAttributes() 
{
	return (methodAttributes);
}
/**
 * This method returns the offset of the method name in the constant pool.
 * @return java.lang.String
 */
public int getMethodName() 
{
	return (methodName);
}
/**
 * This method returns the offset of the method type in the constant pool.
 * @return java.lang.String
 */
public int getMethodType() 
{
	return (methodType);
}
/**
 * This method will save the method table into the file
 * @param outputStream java.io.DataOutputStream
 * @exception java.io.IOException The exception description.
 */
public void save(DataOutputStream outputStream) 
	throws IOException 
{
	outputStream.writeShort(methodMode);
	
	// the offset in the constant pool containing the method name.
	outputStream.writeShort(methodName);

	// the offset in the constant pool containing the method type.
	outputStream.writeShort(methodType);

	// the attributes of the method.
	methodAttributes.save(outputStream);
}
}