//////////////////////////////////////////////////////////////////////////
//
// ALANGUAGE.CPP - Definition of ALanguage, the language selection dialog
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////

#pragma info (none)

#include <ipoint.hpp>
#include <string.h>

#pragma info (restore)

#include "language.hpp"

#include "multilin.hpp"

#include "multilin.rch"

//**************************************************************************
// ALanguage :: ALanguage - Constructor for ALanguage                      *
//**************************************************************************
ALanguage :: ALanguage (IWindow * pOwner)
             : IFrameWindow (DLG_LANGUAGE, pOwner),
               m_pListBox (0L),
               m_pListBoxCursor (0L)
{
  setAutoDeleteObject ();

  ICommandHandler :: handleEventsFor(this); //Set self as command event handler

  setupListBox ();

  center ();

  show ();
}

//**************************************************************************
// ALanguage :: ~ALanguage - Destructor for ALanguage                      *
//**************************************************************************
ALanguage :: ~ALanguage (void)
{
  delete m_pListBox;

  delete m_pListBoxCursor;
}

//**************************************************************************
// ALanguage :: setupListBox - init the language-selection listbox         *
//**************************************************************************
void ALanguage :: setupListBox (void)
{
  m_pListBox = new IListBox (DID_LB_LANGUAGE, this);

  m_pListBox -> addAsFirst ("Language: English");

  m_pListBox -> addAsLast ("Sprache: Deutsch");

  m_pListBox -> select (0);
}

//**************************************************************************
// ABookmarkListDialog :: command - Process Commands                       *
//**************************************************************************
Boolean ALanguage :: command (ICommandEvent& evt)
{
  Boolean bRet = false;

  if(evt. commandId () == DID_OK)
     {
      long lResult = AMultilinTest :: none;
      switch (m_pListBox -> selection ())
              {
               case 0:  // english is selected
                    lResult = AMultilinTest :: english;
                    break;
               case 1:  // german is selected
                    lResult = AMultilinTest :: german;
                    break;
               default:
                    break; // lResult is already set to AMultilinTest :: none
              }
      close ();
      dismiss (lResult);  // set the result to the selected language
      bRet = true;
     }

  return (bRet);
}

//**************************************************************************
// ALanguage :: center - center the window                                 *
//**************************************************************************
void ALanguage :: center (void)
{
  IWindow * pWnd = owner () ? owner () : IWindow :: desktopWindow ();  // check for the owner

  ISize dSize = pWnd -> size ();  // get the size of the window

  IPoint pos = pWnd -> position();  // get the position

  long x = pos.x() + (dSize.width()-size().width())/2;
  long y = pos.y() +(dSize.height()-size().height())/2;

  if (x < 0L)
      x = 0L;

  if (y < 0L)
      y = 0L;

  IPoint location(x, y);

  moveTo (location);                     //Set the position of the window    
}
