///////////////////////////////////////////////////////////////////////////////////////////
//
// MULTILIN.CPP - Description of AMultilinTest, the multilingual resource sample application
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////////////////////

#pragma info (none)

#define INCL_DOSPROCESS
#define INCL_DOSMODULEMGR

#include <os2.h>
#include <imsgbox.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>
#include <ititle.hpp>

#pragma info (restore)

#include "multilin.rch"
#include "multilin.hpp"
#include "about.hpp"
#include "language.hpp"
#include "resdll.hpp"

unsigned long setupLanguage (void);
IString buildExceptionText (const IException& rException);

const char * const AMultilinTest :: m_pszDLLEnglish = "multilie.dll";
const char * const AMultilinTest :: m_pszDLLGerman  = "multilid.dll";

//*************************************************************************
// main  - Application entry point                                        *
//*************************************************************************
int main(int argc, char ** argv)
{
  IApplication :: current(). setArgs (argc, argv);

  unsigned long ulLanguage = setupLanguage ();

  const char * pszDLLName;

  switch (ulLanguage)
          {
           case AMultilinTest :: english:
                pszDLLName = AMultilinTest :: englishDLLName ();
                break;
           case AMultilinTest :: german:
                pszDLLName = AMultilinTest :: germanDLLName ();
                break;
           default:
                pszDLLName = AMultilinTest :: englishDLLName ();
                break;
          }

  try
     {
      if (!AResourceDLL :: loadResourceDLL (pszDLLName))     // try to load the language specific resource DLL
          {
           IMessageBox msgBox (IWindow :: desktopWindow ());
           msgBox .setTitle ("Muiltilin.exe");
           IString strText;
           switch (ulLanguage)
                   {
                    case AMultilinTest :: english:
                         strText = "Can not load ResourceDLL ";
                         strText += AMultilinTest :: englishDLLName ();
                         strText += ".";
                         break;
                    case AMultilinTest :: german:
                         strText = "Kann ResourceDLL ";
                         strText += AMultilinTest :: germanDLLName ();
                         strText += " nicht laden.";
                         break;
                    default:
                         strText = "Can not load ResourceDLL ";
                         strText += AMultilinTest :: englishDLLName ();
                         strText += ".";
                         break;
                   }
           msgBox.show (strText, IMessageBox::cancelButton | IMessageBox::errorIcon
                                 | IMessageBox::applicationModal | IMessageBox::moveable);
           // do not call IApplication :: current (). run (), you can never stop it again
           exit (3);
          }
     }
  catch (IException& rException)  // the parameter paased to AResourceDLL :: loadResourceDLL was wrong
         {
          IMessageBox msgBox (IWindow :: desktopWindow ());
          msgBox. setTitle (rException. name ());
          msgBox.show (buildExceptionText (rException), IMessageBox::cancelButton | IMessageBox::errorIcon
                                                        | IMessageBox::applicationModal
                                                        | IMessageBox::moveable);
          // do not call IApplication :: current (). run (), you can never stop it again
          exit (3);
         }

  AMultilinTest mainWindow (WND_MAIN, ulLanguage);   //Create our main window on the
                                                     // desktop

  IApplication :: current (). run ();                //Get the current application and
                                                     // run it

  return(0);
}

//**************************************************************************
// setupLanguage - global function to set the language                     *
//**************************************************************************
unsigned long setupLanguage (void)
{
  if(IApplication :: current (). argc () > 1)            // we have some runtime arguments
     {
      if(IApplication :: current (). argv (1). lowerCase (). isAbbreviationFor ("-e", 2))
         return (AMultilinTest :: english);          // the user selected english
      else
         if(IApplication :: current (). argv (1). lowerCase (). isAbbreviationFor ("-d", 2))
            return (AMultilinTest :: german);        // the user selected german
     } 

  AIniConfig * pIniConfig = 0L;

  try
     {
      pIniConfig = new AIniConfig ();                  // initialize the initialization file
     }
  catch(IException& rException)                        // any problems 
        {                                              // show the error text
         IMessageBox msgBox (IWindow :: desktopWindow ());
         msgBox. setTitle (rException. name ());
         msgBox.show (buildExceptionText (rException), IMessageBox::okButton
                                                       | IMessageBox::informationIcon
                                                       | IMessageBox::applicationModal
                                                       | IMessageBox::moveable);
        }

  // set the language

  unsigned long ulRet;

  if(pIniConfig                                                // do we have an initialization file?
     && pIniConfig -> isLanguageSaved ()                       // was a value for the language saved?
     && pIniConfig -> getLanguage () != AMultilinTest :: none) // isn't it none?
     {                                                         // yes
      ulRet = pIniConfig -> getLanguage ();                    // set the return-value to the saved language
     }
  else
     {                                                         // no, bring up the language selection dialog
      ALanguage * dlgLanguage = new ALanguage ();
      ulRet = dlgLanguage -> showModally();
      delete dlgLanguage;
     }

  delete pIniConfig;

  return (ulRet);
}

//***************************************************************************
// buildExceptionText - global function to build the exception message text *
//***************************************************************************
IString buildExceptionText (const IException& rException)

{
  IString strRet ("Encountered an exception:\n\n");

  if (!rException. textCount ())
      strRet += "No error text available";
  else
      {
       char szBuffer [21];
       strRet += rException. text (rException. textCount () - 1);
       strRet += " Error ID: ";
       strRet += _ltoa (rException. errorId (), szBuffer, 10);
      }

  return (strRet);
}

//*****************************************************************
// AMultilinTest :: AMultilinTest - Constructor of AMultilinTest  *
//*****************************************************************
AMultilinTest :: AMultilinTest(unsigned long windowId, unsigned long ulCurrentLanguage)
               : IFrameWindow (IFrameWindow::titleBar
                               | IFrameWindow::systemMenu
                               | IFrameWindow::sizingBorder
                               | IFrameWindow::minimizeButton
                               | IFrameWindow::maximizeButton
                               | IFrameWindow::windowList
                               | IFrameWindow::appDBCSStatus
                               | IFrameWindow::minimizedIcon
                               | IFrameWindow::menuBar
                               | IFrameWindow::accelerator,
                               windowId),
                  m_pMenuBar (0L),
                  m_pIniConfig (0L)
{
  setExceptionFunction (&excHandler);

  if (!setupVariables (ulCurrentLanguage))        // setup the variables
      {
       cleanUp ();                                // an error during setupVariables occures, so clean up
       ITHROW (IException ("AMultilinTest :: setupVariables failed", 0)); // and throw an exception
      }

  setAutoDeleteObject ();

  ICommandHandler :: handleEventsFor (this);      // set self as command handler

  setFocus ();                                    // set the focus

  show ();                                        // show the window
}

//*****************************************************************
// AMultilinTest :: ~AMultilinTest - Destructor of AMultilinTest  *
//*****************************************************************
AMultilinTest :: ~AMultilinTest ()

{
  cleanUp ();
}

//*************************************************************************
// AMultilinTest :: englishDLLName - the name of the english resource DLL *
//*************************************************************************
const char * const AMultilinTest :: englishDLLName ()

{
  return (m_pszDLLEnglish);
}

//*************************************************************************
// AMultilinTest :: englishDLLName - the name of the english resource DLL *
//*************************************************************************
const char * const AMultilinTest :: germanDLLName ()

{
  return (m_pszDLLGerman);
}

//**********************************************************************
// AMultilinTest :: cleanUp - clean up of the allocated resources      *
//**********************************************************************
void AMultilinTest :: cleanUp (void)
{
  delete m_pMenuBar;

  delete m_pIniConfig;
}

//*********************************************************************
// AMultilinTest :: setupVariables - variable set up                  * 
//*********************************************************************
Boolean AMultilinTest :: setupVariables (const unsigned long ulCurrentLanguage)
{
  m_pMenuBar = new IMenuBar (WND_MAIN, this);            // initialize the menu bar

  m_pMenuBar -> checkItem (MI_LANG_CHANGE);

  try
     {
      m_pIniConfig = new AIniConfig ();                  // initialize the initialization file
     }
  catch(IException& rException)                        // any problems 
        {                                              // show the error text
         IMessageBox msgBox (IWindow :: desktopWindow ());
         msgBox. setTitle (rException. name ());
         IString strText (buildExceptionText (rException));
         IString strString;
         AResourceDLL :: loadString (strString, STR_CANT_SAVE_SETTINGS, " Can not save changed settings.");
         strText += strString;
         msgBox.show (strText, IMessageBox::okButton
                               | IMessageBox::informationIcon
                               | IMessageBox::applicationModal
                               | IMessageBox::moveable);
        }

  setLanguage (ulCurrentLanguage);

  return (true);
}

//****************************************************************
// AMultilinTest :: setLanguage - switch the language menu items *
//                                and save the language          *
//****************************************************************
AMultilinTest& AMultilinTest :: setLanguage (const unsigned long ulCurrentLanguage)
{
  if(m_pIniConfig)                                    // do we have a valid initialization file?
     m_pIniConfig -> setLanguage (ulCurrentLanguage); // save the startup language

  return (*this);
}

//*********************************************************************
// AMultilinTest :: command - handles Command-Events                 *
//*********************************************************************
Boolean AMultilinTest :: command(ICommandEvent &evt)
{
  Boolean filtered = true;                                        // event handled

  switch(evt.commandId())                      	                  // handle the menu commands
         {
          case MI_FILE_EXIT:                                      // exit the program
               close ();                                          // close the frame window
               break;
          case MI_LANG_CHANGE:
               changeLanguage ();
               break;
          case MI_HELP_ABOUT:                                     // show the product information
               {
                AAbout * pDlgAbout = new AAbout (this);
                pDlgAbout -> showModally ();
                delete pDlgAbout;
                break;
               }
          default:
              filtered = false;                                   // event not handled
              break;
         }

  return filtered;
}

//******************************************************************
// AMultilinTest :: changeLanguage - called to change the language *
//******************************************************************
AMultilinTest& AMultilinTest :: changeLanguage ()
{
  ALanguage * pDlgLanguage = new ALanguage (this);

  unsigned long ulNewLanguage = pDlgLanguage -> showModally ();  // bring up the language selection dialog

  delete pDlgLanguage;

  if (ulNewLanguage == m_pIniConfig -> getLanguage ())           // did the language changed?
      return (*this);                                            // no -> return

  const char * pszDLLName;

  switch (ulNewLanguage)                                         // set the DLL name
          {
           case AMultilinTest :: english:
                pszDLLName = englishDLLName ();
                break;
           case AMultilinTest :: german:
                pszDLLName = germanDLLName ();
                break;
           default:
                pszDLLName = englishDLLName ();
                break;
          }

  IString strText;

  if (!AResourceDLL :: loadResourceDLL (pszDLLName))
      {                                              // can't load the resource DLL
       IMessageBox msgBox (this);
       AResourceDLL :: loadString (strText, STR_CHANGE_LANGUAGE, "Change Language");
       msgBox. setTitle (strText);
       AResourceDLL :: loadString (strText, STR_ERROR_LOAD_DLL, "Can't load the needed resource DLL.");
       msgBox. show (strText, IMessageBox::okButton
                              | IMessageBox::informationIcon
                              | IMessageBox::applicationModal
                              | IMessageBox::moveable);
       return (*this);
      }

  // save the new language
  setLanguage (ulNewLanguage);

  // load the title
  AResourceDLL :: loadString (strText, WND_MAIN, "Multilingual Resource Sample Application");

  // change the title
  ITitle title (this, strText);

  // change the menu
  m_pMenuBar -> setMenu (WND_MAIN);

  return (*this);
}

//*************************************************************************
// AMultilinTest :: ExceptionHandler - default rException handling       *
//*************************************************************************
Boolean AMultilinTest :: ExceptionHandler :: handleException(IException &rException, IEvent &rEvent)
{
  IMessageBox msgBox (rEvent. window ());

  msgBox. setTitle (rException. name ());

  IMessageBox :: Response result =  msgBox.show(buildExceptionText (rException), 
                                                IMessageBox::okCancelButton
                                                | IMessageBox::informationIcon
                                                | IMessageBox::applicationModal
                                                | IMessageBox::moveable);

  return((result == IMessageBox :: ok ? true : false));
}

