///////////////////////////////////////////////////////////////////////////////////////////
//
// MULTILIN.HPP - Declaration of AMultilinTest, the multilingual resource sample application
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
///////////////////////////////////////////////////////////////////////////////////////////

#ifndef _MULTILIN_HPP_
#define _MULTILIN_HPP_

#pragma info (none)

#include <iframe.hpp>
#include <imenubar.hpp>
#include <icmdhdr.hpp>
#include <istattxt.hpp>

#pragma info (restore)

#include "inicfg.hpp"

class AMultilinTest : public IFrameWindow,
                      public ICommandHandler

{
  public:                               
    AMultilinTest (unsigned long windowId, unsigned long ulCurrentLanguage); // Constructor
    ~AMultilinTest ();                                         // Destructor for this class

    enum { none, english, german };

    static const char * const englishDLLName (void);
    static const char * const germanDLLName (void);

  protected:
    void cleanUp (void);                                            // clean up of the allocated resources

    Boolean setupVariables (const unsigned long ulCurrentLanguage); // set up the variables

    AMultilinTest& setLanguage (const unsigned long ulLanguage);    // switch the language menu items
                                                                    // and save the language
    Boolean command(ICommandEvent &evt);                            // command handling

    AMultilinTest& AMultilinTest :: changeLanguage ();

  private:
    IMenuBar * m_pMenuBar;                                    // pointer to the menu bar
    AIniConfig * m_pIniConfig;                                // pointer to the initialization file

    static const char * const m_pszDLLEnglish;
    static const char * const m_pszDLLGerman;

    // default exception handler class
    class ExceptionHandler : public IWindow::ExceptionFn
        {
         public:
           virtual Boolean handleException(IException &exception, 
                                           IEvent     &rEvent);
        };

    ExceptionHandler excHandler;
};


#endif //_MULTILIN_HPP_
