/////////////////////////////////////////////////////////////////////////////
//
// RESDLL.CPP - Definition of AResourceDLL, a helper class for resource DLLs
//
// (c) 1997 by Stefan Ruck, Frankfurt/Main, Germany
//
//////////////////////////////////////////////////////////////////////////////

#pragma info (none)

#define INCL_DOSPROCESS
#define INCL_DOSMODULEMGR

#include <os2.h>
#include <iapp.hpp>
#include <ireslib.hpp>
#include <istring.hpp>

#pragma info (restore)

#include "resdll.hpp"

//**********************************************************
// AResourceDLL :: loadResourceDLL - load the resource DLL *
//                                                         *
// throws IAssertationFailure                              *
//**********************************************************
Boolean AResourceDLL :: loadResourceDLL (const char * pszDLLName)

{
  if (!pszDLLName)
      IException :: assertParameter ("pszDLLName is NULL!", 
                                     IExceptionLocation ("resdll.cpp", 
                                                         "AResourceDLL :: loadResourceDLL", 28));

  IString strDLLName (pszDLLName);

  strDLLName. upperCase ();

  strDLLName. remove (strDLLName. lastIndexOf (".DLL"));                // remove the filename suffix

  try
     {
      IApplication :: current (). setUserResourceLibrary (strDLLName);  // try to load the DLL from
     }                                                                  // the current directory (or path)

  catch (IException&)
         {
          return (AResourceDLL :: loadResDLLFromAppDir (strDLLName));   // try to load the resource from 
         }                                            // the app's dir and return loadResDllFromAppDir's 
                                                      // return value
  return (true);
}

//*********************************************************************
// AResourceDLL :: loadResDLLFromAppDir - load the resource DLL from  *
//                                        the application's directory *
//                                                                    *
// throws IAssertationFailure                                         *
//*********************************************************************
Boolean AResourceDLL :: loadResDLLFromAppDir (const char * pszDLLName)

{
  if (!pszDLLName)
      IException :: assertParameter ("pszDLLName is NULL!", 
                                     IExceptionLocation ("resdll.cpp", 
                                                         "AResourceDLL :: loadResDLLFromAppDir", 52));

  PPIB pPib = NULL;
  PTIB pTib = NULL;

  DosGetInfoBlocks (&pTib, &pPib);  // get the info blocks

  char pszFilename [CCHMAXPATH];

  if (DosQueryModuleName (pPib -> pib_hmte, sizeof (pszFilename), pszFilename)) // can't query module name
      return (false);                                                           // return (false)

  IString strFilename = pszFilename;  // keep the module name

  strFilename. remove (strFilename. lastIndexOf ('\\')); // remove the filename, leaves the path

  strFilename += "\\";                                   // add the \

  strFilename += pszDLLName;                             // add the DLL name

  strFilename. upperCase ();

  if (!strFilename. lastIndexOf (".DLL"))                // do we have the DLL suffix
      strFilename += ".DLL";                             // no, so add it

  try
     {
      IApplication :: current (). setUserResourceLibrary (strFilename); // try to load it
     }
  catch(IException&)
        {
         return (false);  // can't load the resource DLL, so return (false)
        }

  return (true);          // resource DLL successfully loaded
}

//************************************************************************
// AResourceDLL :: loadString - loads a string from the resource library *
//************************************************************************
void AResourceDLL :: loadString (IString& dest, const unsigned long ulResourceID, const char * pszText)

{
  try
     {
      dest = IApplication :: current (). userResourceLibrary (). loadString (ulResourceID);
     }

  catch (IException&)
         {
          dest = pszText;
         }
}
