
package shai.security.demo;

import java.security.*;
import java.io.*;

/**
 * This class was generated by a SmartGuide.
 * 
 */
public class Demo 
{
	// this is the data contained within the file.
	byte fileData[];
	
	// this is the signature.
	private byte sigData[];
	
	// this is the object used to generate the signature.
	private Signature sig;
	
	// this is the object representing the pair of keys.
	private KeyPair keys;

	public Demo(String algorithm)
	{
		try
		{
			sig = Signature.getInstance(algorithm);
			KeyPairGenerator keyGen = KeyPairGenerator.getInstance(algorithm);
			keyGen.initialize(1024);
			keys = keyGen.generateKeyPair();
			
		}	
		catch(NoSuchAlgorithmException err)
		{
			System.out.println("the algorithm " + algorithm + " is not supported: " + err.getMessage());
		}	
	}	
	public static void main(String argv[])
	{
		if(argv.length == 2)
		{
			Demo program = new Demo(argv[0]);
			program.sign(argv[1]);
			program.verifySignature();
		}	
		else
			System.out.println("This program takes two parameters:\n" 
					+ " The signiture algorithm used and the file name to use it on.");
	}
/**
 * This method signs the file fileName.
 * @param fileName java.lang.String
 */
public void sign(String fileName) 
{
	try
	{
		// initialize the signiture object to the private key.
		sig.initSign(keys.getPrivate());

		// create a file object to connect to the file.
		File fileToSign = new File(fileName);
		
		// if the file to sign is not OK throw an exception
		if(!fileToSign.isFile())
			throw(new IOException("File " + fileName + " not found."));

		// read in the data contained in the file we wish to sign.
		fileData = new byte[(int)fileToSign.length()];
		FileInputStream fileStream = new FileInputStream(fileToSign);
		fileStream.read(fileData);
		fileStream.close();
		
		// generate a signature from the data in the file.
		sig.update(fileData);
	
		// save the signature into memory.	
		sigData = sig.sign();
	}
	catch(InvalidKeyException err)
	{
		System.out.println("Invalid key! " + err.getMessage());
	}	
	catch(SignatureException sigerr)
	{
		System.out.println("Signing failed: " + sigerr.getMessage());
	}	
	catch (IOException ioerr)
	{
		System.out.println("IO error! " + ioerr.getMessage());
	}	
}
/**
 * This method verifies that the data and the signature are correct.
 */
public void verifySignature() 
{
	try
	{
		// this is the same signatue object used for signing but it does not
		// have to be this way. It is simply more convinient.
		sig.initVerify(keys.getPublic());
	
		sig.update(fileData);
		
		if(sig.verify(sigData))
			System.out.println("Signing succeeded. The signature is valid.");
		else
			System.out.println("Signing failed the signature did not work out.");
	}		
	catch(InvalidKeyException err)
	{
		System.out.println("Invalid key! " + err.getMessage());
	}	
	catch(SignatureException sigerr)
	{
		System.out.println("Signing failed: " + sigerr.getMessage());
	}	
}
}